/*
 * Copyright (c) 2011 David Yat Sin <dyatsin@sangoma.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms|with or without
 * modification|are permitted provided that the following conditions
 * are met:
 *
 * * Redistributions of source code must retain the above copyright
 * notice|this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 * notice|this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * * Neither the name of the original author; nor the names of any contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES|INCLUDING|BUT NOT
 * LIMITED TO|THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT|INDIRECT|INCIDENTAL|SPECIAL,
 * EXEMPLARY|OR CONSEQUENTIAL DAMAGES (INCLUDING|BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE|DATA|OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY|WHETHER IN CONTRACT|STRICT LIABILITY|OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE|EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* INCLUDE ********************************************************************/
#include "ftmod_sangoma_ss7_main.h"

void sngss7_mtp2api_dat_ind(int16_t suId, void *data, int16_t len)
{
	void *mydata;
	ftdm_sigmsg_t	sig;
	ftdm_channel_t *ftdmchan = NULL;	
	
	SS7_FUNC_TRACE_ENTER(__FUNCTION__);

	ftdm_assert(len > 0, "Received data with invalid length\n");

	if (!(g_ftdm_sngss7_data.cfg.mtp3LiLink[suId].flags & SNGSS7_CONFIGURED)) {
		ftdm_log(FTDM_LOG_CRIT, "Received CON CFM on unconfigued MTP3 LI LINK\n");
		SS7_FUNC_TRACE_EXIT(__FUNCTION__);
		return;
	}

	ftdm_assert(g_ftdm_sngss7_data.cfg.mtp3LiLink[suId].ftdmchan, "MTP3 LI Link does not have a ftdmchan!!");

	ftdmchan = g_ftdm_sngss7_data.cfg.mtp3LiLink[suId].ftdmchan;

	mydata = ftdm_malloc(len);
	ftdm_assert(mydata, "Failed to allocate memory\n");	
	memcpy(mydata, data, len);

	memset(&sig, 0, sizeof(sig));
	sig.chan_id = ftdmchan->chan_id;
	sig.span_id = ftdmchan->span_id;
	sig.channel = ftdmchan;
	sig.event_id = FTDM_SIGEVENT_IO_INDATA;

	ftdm_sigmsg_set_raw_data(&sig, mydata, len);

	if (ftdm_span_send_signal(ftdmchan->span, &sig) != FTDM_SUCCESS) {
		ftdm_log_chan_msg(ftdmchan, FTDM_LOG_CRIT, "Failed to send event to user \n");
	}

	SS7_FUNC_TRACE_EXIT(__FUNCTION__);
	return;
}

void sngss7_mtp2api_dat_cfm(int16_t suId, void *data, int16_t len, MtpStatus status, SeqU24 credit)
{
	/* Description:
	 * MTP2 invokes this function to return transmitted but unacknowledged data to Layer 3. Layer 3 needs
	 * to send SdUiStdStaReq to retrieve unacknowledged messages from MTP2 during changeover procedure.
	 * 
	 * status = DL_TX_NACK_MORE = more data buffers available
	 * status = DL_TX_NACK_NO_MORE = this is the last data buffer
	 * 
	 * credit = not used by MTP 2
	 */	
	void *mydata;
	ftdm_sigmsg_t	sig;
	ftdm_channel_t *ftdmchan = NULL;
	
	SS7_FUNC_TRACE_ENTER(__FUNCTION__);

	ftdm_assert(len > 0, "Received data with invalid length\n");

	if (!(g_ftdm_sngss7_data.cfg.mtp3LiLink[suId].flags & SNGSS7_CONFIGURED)) {
		ftdm_log(FTDM_LOG_CRIT, "Received CON CFM on unconfigued MTP3 LI LINK\n");
		SS7_FUNC_TRACE_EXIT(__FUNCTION__);
		return;
	}

	ftdm_assert(g_ftdm_sngss7_data.cfg.mtp3LiLink[suId].ftdmchan, "MTP3 LI Link does not have a ftdmchan!!");

	ftdmchan = g_ftdm_sngss7_data.cfg.mtp3LiLink[suId].ftdmchan;

	mydata = ftdm_malloc(len);
	ftdm_assert(mydata, "Failed to allocate memory\n");	
	memcpy(mydata, data, len);

	memset(&sig, 0, sizeof(sig));
	sig.chan_id = ftdmchan->chan_id;
	sig.span_id = ftdmchan->span_id;
	sig.channel = ftdmchan;
	if (status == DL_TX_NACK_MORE) {
		sig.ev_data.raw_id = SNGSS7_IND_DATA_MORE;
	} else {
		sig.ev_data.raw_id = SNGSS7_IND_DATA_NO_MORE;
	}
	
	sig.event_id = FTDM_SIGEVENT_RAW;

	ftdm_sigmsg_set_raw_data(&sig, mydata, len);

	if (ftdm_span_send_signal(ftdmchan->span, &sig) != FTDM_SUCCESS) {
		ftdm_log_chan_msg(ftdmchan, FTDM_LOG_CRIT, "Failed to send event to user \n");
	}

	SS7_FUNC_TRACE_EXIT(__FUNCTION__);
	return;
}

void sngss7_mtp2api_con_cfm(int16_t suId)
{
	sngss7_chan_data_t *sngss7_info = NULL;
	
	SS7_FUNC_TRACE_ENTER(__FUNCTION__);
	/* This function informs upper layer that the connection
	 * has succeeded and that the link is now connected */

	if (!(g_ftdm_sngss7_data.cfg.mtp3LiLink[suId].flags & SNGSS7_CONFIGURED)) {
		ftdm_log(FTDM_LOG_CRIT, "Received CON CFM on unconfigued MTP3 LI LINK\n");
		SS7_FUNC_TRACE_EXIT(__FUNCTION__);
		return;
	}

	ftdm_assert(g_ftdm_sngss7_data.cfg.mtp3LiLink[suId].ftdmchan, "MTP3 LI Link does not have a ftdmchan!!");
		
	sngss7_info = (sngss7_chan_data_t*)g_ftdm_sngss7_data.cfg.mtp3LiLink[suId].ftdmchan->call_data;
	ftdm_assert(sngss7_info, "Channel does not have link information!\n");
	
	sngss7_set_sig_status(sngss7_info, FTDM_SIG_STATE_UP, 0);
	
	SS7_FUNC_TRACE_EXIT(__FUNCTION__);
	return;
}

void sngss7_mtp2api_disc_ind(uint16_t suId, Reason reason)
{
	uint8_t user_reason = 0;
	sngss7_chan_data_t *sngss7_info = NULL;
	
	SS7_FUNC_TRACE_ENTER(__FUNCTION__);
	/* This function informs upper layer that the MTP2 Layer connection
	* has failed */

	if (!(g_ftdm_sngss7_data.cfg.mtp3LiLink[suId].flags & SNGSS7_CONFIGURED)) {
		ftdm_log(FTDM_LOG_CRIT, "Received DISC IND on unconfigued MTP3 LI LINK\n");
		SS7_FUNC_TRACE_EXIT(__FUNCTION__);
		return;
	}

	ftdm_assert(g_ftdm_sngss7_data.cfg.mtp3LiLink[suId].ftdmchan, "MTP3 LI Link does not have a ftdmchan!!");

	sngss7_info = (sngss7_chan_data_t*)g_ftdm_sngss7_data.cfg.mtp3LiLink[suId].ftdmchan->call_data;
	ftdm_assert(sngss7_info, "Channel does not have link information!\n");

	switch (reason) {
		case DL_DISC_SM:
			user_reason = SNGSS7_REASON_SM;
			break;
		case DL_DISC_SUERM:
			user_reason = SNGSS7_REASON_SUERM;
			break;
		case DL_DISC_ACK:
			user_reason = SNGSS7_REASON_ACK;
			break;
		case DL_DISC_TE:
			user_reason = SNGSS7_REASON_TE;
			break;
		case DL_DISC_BSN:
			user_reason = SNGSS7_REASON_BSN;
			break;
		case DL_DISC_FIB:
			user_reason = SNGSS7_REASON_FIB;
			break;
		case DL_DISC_CONG:
			user_reason = SNGSS7_REASON_CONG;
			break;
		case DL_DISC_LSSU_SIOS:
			user_reason = SNGSS7_REASON_LSSU_SIOS;
			break;
		case DL_DISC_TMR2_EXP:
			user_reason = SNGSS7_REASON_TMR2_EXP;
			break;
		case DL_DISC_TMR3_EXP:
			user_reason = SNGSS7_REASON_TMR3_EXP;
			break;
		case DL_DISC_LSSU_SIOS_IAC:
			user_reason = SNGSS7_REASON_LSSU_SIOS_IAC;
			break;
		case DL_DISC_PROV_FAIL:
			user_reason = SNGSS7_REASON_PROV_FAIL;
			break;
		case DL_DISC_TMR1_EXP:
			user_reason = SNGSS7_REASON_TMR1_EXP;
			break;
		case DL_DISC_LSSU_SIN:
			user_reason = SNGSS7_REASON_LSSU_SIN;
			break;
		case DL_DISC_CTS_LOST:
			user_reason = SNGSS7_REASON_CTS_LOST;
			break;
		case DL_DISC_DAT_IN_OOS:
			user_reason = SNGSS7_REASON_DAT_IN_OOS;
			break;
		case DL_DISC_DAT_IN_WAITFLUSHCONT:
			user_reason = SNGSS7_REASON_DAT_IN_WAITFLUSHCONT;
			break;
		case DL_DISC_RETRV_IN_INS:
			user_reason = SNGSS7_REASON_RETRV_IN_INS;
			break;
		case DL_DISC_CON_IN_INS:
			user_reason = SNGSS7_REASON_CON_IN_INS;
			break;
		case DL_DISC_UPPER_SAP_DIS:
			user_reason = SNGSS7_REASON_UPPER_SAP_DIS;
			break;
		default:
			ftdm_log(FTDM_LOG_CRIT, "Unknown DISC reason received: %d\n", reason);
			break;
	}

	/* Max possible value for reason is 27 */
	sngss7_set_sig_status(sngss7_info, FTDM_SIG_STATE_DOWN, user_reason);
	
	SS7_FUNC_TRACE_EXIT(__FUNCTION__);
	return;
}

void sngss7_mtp2api_sta_ind(uint16_t suId, MtpStatus status)
{
	sngss7_chan_data_t *sngss7_info = NULL;
	ftdm_signaling_status_t sigstatus = FTDM_SIG_STATE_DOWN;
	uint8_t user_reason = 0;
	
	SS7_FUNC_TRACE_ENTER(__FUNCTION__);
	/* Description:
	* MTP2 invokes this function to indicate the status of the remote signalling point.
	*
	* status = DL_REM_PROC_DN = Remote processor down
	* status = DL_REM_PROC_UP = Remote processor up
	*/

	if (!(g_ftdm_sngss7_data.cfg.mtp3LiLink[suId].flags & SNGSS7_CONFIGURED)) {
		ftdm_log(FTDM_LOG_CRIT, "Received DISC IND on unconfigued MTP3 LI LINK\n");
		SS7_FUNC_TRACE_EXIT(__FUNCTION__);
		return;
	}

	ftdm_assert(g_ftdm_sngss7_data.cfg.mtp3LiLink[suId].ftdmchan, "MTP3 LI Link does not have a ftdmchan!!");

	sngss7_info = (sngss7_chan_data_t*)g_ftdm_sngss7_data.cfg.mtp3LiLink[suId].ftdmchan->call_data;
	ftdm_assert(sngss7_info, "Channel does not have link information!\n");

	switch(status) {
		case DL_REM_PROC_DN:
			sigstatus = FTDM_SIG_STATE_SUSPENDED;
			user_reason = SNGSS7_REASON_REM_PROC_DN;
			break;
		case DL_REM_PROC_UP:
			sigstatus = FTDM_SIG_STATE_UP;
			user_reason = SNGSS7_REASON_REM_PROC_UP;
			break;

	}
	sngss7_set_sig_status(sngss7_info, sigstatus, user_reason);
	SS7_FUNC_TRACE_EXIT(__FUNCTION__);
	return;
}

void sngss7_mtp2api_sta_cfm(uint16_t suId, Action action, SeqU24 status)
{
	ftdm_channel_t *ftdmchan;
	uint8_t user_reason = 0;
	sngss7_chan_data_t *sngss7_info = NULL;
	ftdm_signaling_status_t sigstatus = FTDM_SIG_STATE_DOWN;
	
	SS7_FUNC_TRACE_ENTER(__FUNCTION__);
	if (!(g_ftdm_sngss7_data.cfg.mtp3LiLink[suId].flags & SNGSS7_CONFIGURED)) {
		ftdm_log(FTDM_LOG_CRIT, "Received DISC IND on unconfigued MTP3 LI LINK\n");
		SS7_FUNC_TRACE_EXIT(__FUNCTION__);
		return;
	}

	ftdm_assert(g_ftdm_sngss7_data.cfg.mtp3LiLink[suId].ftdmchan, "MTP3 LI Link does not have a ftdmchan!!");
	ftdmchan = g_ftdm_sngss7_data.cfg.mtp3LiLink[suId].ftdmchan;

	sngss7_info = (sngss7_chan_data_t*) ftdmchan->call_data;
	ftdm_assert(sngss7_info, "Channel does not have link information!\n");

	switch(action) {
		case DL_RETRV_BSN:
			{
				uint16_t *mydata;
				ftdm_sigmsg_t	sig;
				
				memset(&sig, 0, sizeof(sig));
				sig.chan_id = ftdmchan->chan_id;
				sig.span_id = ftdmchan->span_id;
				sig.channel = ftdmchan;

				sig.ev_data.raw_id = SNGSS7_IND_RETRV_BSN;
				sig.event_id = FTDM_SIGEVENT_RAW;

				mydata = ftdm_malloc(sizeof(*mydata));

				ftdm_assert(mydata, "Failed to allocate memory");

				*mydata = status;

				ftdm_sigmsg_set_raw_data(&sig, mydata, sizeof(*mydata));

				if (ftdm_span_send_signal(ftdmchan->span, &sig) != FTDM_SUCCESS) {
					ftdm_log_chan_msg(ftdmchan, FTDM_LOG_CRIT, "Failed to send event to user \n");
				}
			}
			break;
		case DL_RETRV_EMPTY:
			{
				/* User requested to retrieve messages, but no messages in buffer */
				ftdm_sigmsg_t	sig;
				memset(&sig, 0, sizeof(sig));
				sig.chan_id = ftdmchan->chan_id;
				sig.span_id = ftdmchan->span_id;
				sig.channel = ftdmchan;
				sig.ev_data.raw_id = SNGSS7_IND_DATA_EMPTY;

				sig.event_id = FTDM_SIGEVENT_RAW;
				if (ftdm_span_send_signal(ftdmchan->span, &sig) != FTDM_SUCCESS) {
					ftdm_log_chan_msg(ftdmchan, FTDM_LOG_CRIT, "Failed to send event to user \n");
				}
			}
			break;
		case DL_FLC_STAT:
			switch(status) {
				case DL_STARTFLC:
					sigstatus = FTDM_SIG_STATE_SUSPENDED;
					user_reason = SNGSS7_REASON_START_FLC;
					break;
				case DL_ENDFLC:
					sigstatus = FTDM_SIG_STATE_UP;
					user_reason = SNGSS7_REASON_END_FLC;
					break;
				default:
					ftdm_log_chan(ftdmchan, FTDM_LOG_CRIT, "Received invalid STA CFM FLC status %d\n", status);
					return;
			}
			sngss7_set_sig_status(sngss7_info, sigstatus, user_reason);
			break;
		default:
			ftdm_log_chan(ftdmchan, FTDM_LOG_CRIT, "Received invalid STA CFM action %d\n", action);
			break;

	}
	SS7_FUNC_TRACE_EXIT(__FUNCTION__);
	return;
}

void sngss7_mtp2api_flc_ind(uint16_t suId, Action action)
{
	sngss7_chan_data_t *sngss7_info = NULL;
	ftdm_signaling_status_t sigstatus = FTDM_SIG_STATE_DOWN;
	uint8_t user_reason = 0;
	
	SS7_FUNC_TRACE_ENTER(__FUNCTION__);

	if (!(g_ftdm_sngss7_data.cfg.mtp3LiLink[suId].flags & SNGSS7_CONFIGURED)) {
		ftdm_log(FTDM_LOG_CRIT, "Received DISC IND on unconfigued MTP3 LI LINK\n");
		SS7_FUNC_TRACE_EXIT(__FUNCTION__);
		return;
	}

	ftdm_assert(g_ftdm_sngss7_data.cfg.mtp3LiLink[suId].ftdmchan, "MTP3 LI Link does not have a ftdmchan!!");

	sngss7_info = (sngss7_chan_data_t*)g_ftdm_sngss7_data.cfg.mtp3LiLink[suId].ftdmchan->call_data;
	ftdm_assert(sngss7_info, "Channel does not have link information!\n");

	switch(action) {
		case DL_STARTFLC:
			sigstatus = FTDM_SIG_STATE_SUSPENDED;
			user_reason = SNGSS7_REASON_START_FLC;
			break;
		case DL_ENDFLC:
			sigstatus = FTDM_SIG_STATE_UP;
			user_reason = SNGSS7_REASON_END_FLC;
			break;

	}
	sngss7_set_sig_status(sngss7_info, sigstatus, user_reason);
	SS7_FUNC_TRACE_EXIT(__FUNCTION__);
	return;
}

ftdm_status_t ftmod_sangoma_ss7_mtp2_set_sig_status(ftdm_channel_t *ftdmchan, ftdm_signaling_status_t status)
{
	ftdm_status_t ret = FTDM_SUCCESS;
	sngss7_chan_data_t *sngss7_info;
	
	SuId suId;
	SS7_FUNC_TRACE_ENTER(__FUNCTION__);
	sngss7_info = (sngss7_chan_data_t*) ftdmchan->call_data;
	suId = sngss7_info->api_data.mtp2_id;

	ftdm_assert(ftdmchan->usrmsg, "Channel does not have user data attached\n");
	switch(status) {
		case FTDM_SIG_STATE_UP:
			switch(ftdmchan->usrmsg->raw_id) {
				case SNGSS7_SIGSTATUS_NORM:
					ftmod_ss7_activate_mtp3lilink(suId, DL_CON_L1L2_NORM);
					break;
				case SNGSS7_SIGSTATUS_EMERG:
					ftmod_ss7_activate_mtp3lilink(suId, DL_CON_L1L2_EMRG);
					break;
				case SNGSS7_SIGSTATUS_FLC_STAT:
					sng_ss7_ap_sta_req(suId, DL_FLC_STAT, 0);
					break;
				case SNGSS7_SIGSTATUS_LOC_PROC_UP:
					sng_ss7_ap_sta_req(suId, DL_NO_ACTION, DL_LOC_PROC_UP);
					break;
				default:
					ftdm_log_chan(ftdmchan, FTDM_LOG_ERROR, "Invalid sigstatus ID for UP (%d)\n", ftdmchan->usrmsg->raw_id);
					ret = FTDM_FAIL;
					break;
			}
			break;
		case FTDM_SIG_STATE_SUSPENDED:
			switch(ftdmchan->usrmsg->raw_id) {
				case SNGSS7_SIGSTATUS_LOC_PROC_DN:
					sng_ss7_ap_sta_req(suId, DL_NO_ACTION, DL_LOC_PROC_DN);
					break;
				default:
					ftdm_log_chan(ftdmchan, FTDM_LOG_ERROR, "Invalid sigstatus ID for SUSPEND (%d)\n", ftdmchan->usrmsg->raw_id);
					ret = FTDM_FAIL;
					break;
			}
			break;
		default:			
			ret = FTDM_FAIL;
	}
	
	SS7_FUNC_TRACE_EXIT(__FUNCTION__);
	return ret;
}

ftdm_status_t ftmod_sangoma_ss7_mtp2_transmit(ftdm_channel_t *ftdmchan, void *data, ftdm_size_t size)
{
	sngss7_chan_data_t *sngss7_info;
	
	SuId suId;
	SS7_FUNC_TRACE_ENTER(__FUNCTION__);
	sngss7_info = (sngss7_chan_data_t*) ftdmchan->call_data;
	suId = sngss7_info->api_data.mtp2_id;

	sng_ss7_ap_dat_req(suId, data, size);
	SS7_FUNC_TRACE_EXIT(__FUNCTION__);
	return FTDM_BREAK;
}

/* This function is called by MTP2 User App to request list of unacknowledged frames */
ftdm_status_t ftmod_sangoma_ss7_mtp2_indicate(ftdm_channel_t *ftdmchan)
{
	SuId suId;
	sngss7_chan_data_t *ss7_info;
	Action action = 0;
	MtpStatus status = 0;
	int16_t ret = 0;
	
	SS7_FUNC_TRACE_ENTER(__FUNCTION__);

	ss7_info = (sngss7_chan_data_t*)ftdmchan->call_data;	
	suId = ss7_info->api_data.mtp2_id;
	
	ftdm_assert(ftdmchan->usrmsg, "Channel does not have user data attached\n");
	switch(ftdmchan->usrmsg->raw_id) {
		case SNGSS7_REQ_FLUSH_BUFFERS:
			action = DL_NO_ACTION;
			status = DL_FLUSH_BUF;
			break;
		case SNGSS7_REQ_CONTINUE:
			action = DL_NO_ACTION;
			status = DL_CONTINUE;
			break;
		case SNGSS7_REQ_RETRV_BSN:
			action = DL_RETRV_BSN;
			break;
		case SNGSS7_REQ_RETRV_MSG:
			{
				/* BSN is stored in usrmsg */
				ftdm_size_t len;
				void *vdata;
				
				if (ftdm_usrmsg_get_raw_data(ftdmchan->usrmsg, &vdata, &len) != FTDM_SUCCESS) {
					ftdm_log_chan_msg(ftdmchan, FTDM_LOG_CRIT, "Failed to retrieve user raw data\n");
					return FTDM_FAIL;
				}

				if (len != 2) {
					ftdm_log_chan_msg(ftdmchan, FTDM_LOG_CRIT, "Invalid raw data lengh for BSN\n");
					return FTDM_FAIL;
				}
				action = DL_RETRV_MSG;
				status = *((uint16_t*)vdata);
			}
			break;
		case SNGSS7_REQ_DROP_MSGQ:
			action = DL_DROP_MSGQ;
			break;
	}

	ret = sng_ss7_ap_sta_req(suId, action, status);
	SS7_FUNC_TRACE_EXIT(__FUNCTION__);
	if (!ret) {
		return FTDM_SUCCESS;
	} else {
		return FTDM_FAIL;
	}
	
}



